import tkinter as tk
from tkinter import ttk
import numpy as np
import matplotlib.pyplot as plt
import sounddevice as sd

def generate_noise():
    frequencies = [float(freq1_var.get())]
    waveform_types = [wave1_type_var.get()]

    # Sample rate and time array
    sample_rate = 44100
    duration = float(duration_entry.get())
    t = np.linspace(0, duration, int(sample_rate * duration), endpoint=False)

    # Generate overlapping sounds
    combined_wave = np.zeros_like(t)
    for frequency, waveform_type in zip(frequencies, waveform_types):
            if waveform_type == "Sine":
                wave = np.sin(2 * np.pi * frequency * t)
            elif waveform_type == "Square":
                wave = np.sign(np.sin(2 * np.pi * frequency * t))
            elif waveform_type == "Triangle":
                wave = 2 * np.abs(2 * (t * frequency - np.floor(t * frequency + 0.5)))
            else:
                wave = np.random.uniform(-1, 1, len(t))

            combined_wave += wave

    # Play the combined sound using sounddevice
    sd.play(combined_wave, sample_rate)

def plot_one_period_combined_wave(t, combined_wave):
    # Find the index corresponding to the end of one period
    period_end_index = np.argmax(t >= 1 / min([float(freq1_var.get())]))

    plt.figure()
    plt.plot(t[:5 * period_end_index], combined_wave[:5 * period_end_index])
    plt.title("Five Periods of Sound")
    plt.xlabel('Time (s)')
    plt.ylabel('Amplitude')
    plt.grid()
    plt.show()

def update_sound_type(wave_type_var, waveform_type):
    # Update label to show the current waveform type
    wave_type_var.config(text=f"Waveform Type: {waveform_type}")
def plot_function():
    frequencies = [float(freq1_var.get())]
    waveform_types = [wave1_type_var.get()]

    # Sample rate and time array
    sample_rate = 44100
    duration = float(duration_entry.get())
    t = np.linspace(0, duration, int(sample_rate * duration), endpoint=False)

    # Generate overlapping sounds
    combined_wave = np.zeros_like(t)
    for frequency, waveform_type in zip(frequencies, waveform_types):
      #  if enabled:
            if waveform_type == "Sine":
                wave = np.sin(2 * np.pi * frequency * t)
            elif waveform_type == "Square":
                wave = np.sign(np.sin(2 * np.pi * frequency * t))
            elif waveform_type == "Triangle":
                wave = 2 * np.abs(2 * (t * frequency - np.floor(t * frequency + 0.5)))
            else:
                wave = np.random.uniform(-1, 1, len(t))

            combined_wave += wave

    # Plot one period of the combined sound
    plot_one_period_combined_wave(t, combined_wave)


def exit_application():
    root.destroy()

    # Call the generate_noise function to update the sound
    generate_noise()

root = tk.Tk()
root.title("Sound Generator")

# Waveform 1
frame_wave1 = ttk.LabelFrame(root, text="Waveform 1")
frame_wave1.pack(side="left", padx=10, pady=10)

wave1_type_var = tk.StringVar(value="Sine")
wave1_type_label = tk.Label(frame_wave1, text="Waveform Type:")
wave1_type_label.pack()
wave1_type_dropdown = ttk.Combobox(frame_wave1, textvariable=wave1_type_var, values=["Sine", "Square", "Triangle", "Random"])
wave1_type_dropdown.pack()

freq1_var = tk.DoubleVar(value=440.0)
freq1_label = tk.Label(frame_wave1, text="Frequency:")
freq1_label.pack()
freq1_slider = tk.Scale(frame_wave1, variable=freq1_var, from_=0, to=2000, orient="horizontal", length=200,
                        command=lambda value: update_sound_type(wave1_type_label, wave1_type_var.get()))
freq1_slider.pack()

# Duration entry
duration_label = tk.Label(root, text="Duration (seconds):")
duration_label.pack()
duration_entry = tk.Entry(root)
duration_entry.insert(0, "5.0")
duration_entry.pack()

# Generate button
generate_button = tk.Button(root, text="Generate and Play", command=generate_noise)
generate_button.pack()

# Plot Function button
plot_function_button = tk.Button(root, text="Plot Function", command=plot_function)
plot_function_button.pack()

# Exit button
exit_button = tk.Button(root, text="Exit", command=exit_application)
exit_button.pack()



root.mainloop()

